function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
import useKeyboardEvents from '@acusti/use-keyboard-events';
import * as React from 'react';
import EventPropsContext from './EventPropsContext.js';
import SubscribeContext from './SubscribeContext.js';
const {
  forwardRef,
  useCallback,
  useRef,
  useState
} = React;
const ReactHigherEventProvider = forwardRef(({
  children,
  component,
  ...extraProps
}, ref) => {
  const eventsRef = useRef(new Map());
  const lastNativeEventRef = useRef(null);
  const keyDownHandlersRef = useRef(new Set());
  const keyPressHandlersRef = useRef(new Set());
  const keyUpHandlersRef = useRef(new Set());
  const [eventProps, setEventProps] = useState({});
  const getEventHandlers = useCallback(eventName => {
    if (eventName === 'onKeyDown') return keyDownHandlersRef.current;
    if (eventName === 'onKeyPress') return keyPressHandlersRef.current;
    if (eventName === 'onKeyUp') return keyUpHandlersRef.current;
    let eventHandlers = eventsRef.current.get(eventName);
    if (!eventHandlers) {
      eventHandlers = new Set();
      eventsRef.current.set(eventName, eventHandlers);
    }
    return eventHandlers;
  }, []);
  const dispatchEvent = useCallback((eventName, event) => {
    const events = eventsRef.current;
    const handlers = events.get(eventName);
    // Check nativeEvent to make sure we haven’t already handled this event
    if (!handlers || event.nativeEvent === lastNativeEventRef.current) {
      return;
    }
    lastNativeEventRef.current = event.nativeEvent;
    handlers.forEach(handler => handler(event));
  }, []);
  useKeyboardEvents({
    onKeyDown: event => {
      keyDownHandlersRef.current.forEach(handler => handler(event));
    },
    onKeyPress: event => {
      keyPressHandlersRef.current.forEach(handler => handler(event));
    },
    onKeyUp: event => {
      keyUpHandlersRef.current.forEach(handler => handler(event));
    }
  });
  const subscribe = useCallback((eventName, handler) => {
    const isKeyboardEvent = eventName.startsWith('onKey');
    const eventHandlers = getEventHandlers(eventName);
    eventHandlers.add(handler);
    setEventProps(props => {
      if (isKeyboardEvent || props[eventName]) return props;
      return {
        ...props,
        [eventName]: event => dispatchEvent(eventName, event)
      };
    });

    // Return the cleanup function
    return () => {
      const handlers = getEventHandlers(eventName);
      handlers.delete(handler);
      // If there are no more handlers for this event, remove it from eventProps
      if (!isKeyboardEvent && !handlers.size) {
        setEventProps(props => {
          if (!props[eventName]) return props;
          const {
            [eventName]: deletedEvent,
            ...nextProps
          } = props;
          return nextProps;
        });
      }
    };
  }, []);
  const Component = component || 'div';
  return (
    /*#__PURE__*/
    // $FlowExpectedError cannot-spread-indexer from spreading eventProps
    React.createElement(Component, _extends({}, extraProps, eventProps, {
      ref: ref
    }), /*#__PURE__*/React.createElement(EventPropsContext.Provider, {
      value: eventProps
    }, /*#__PURE__*/React.createElement(SubscribeContext.Provider, {
      value: subscribe
    }, children)))
  );
});
export default ReactHigherEventProvider;